@@ -604,7 +604,10 @@ static int verify_negotiate_response(struct auth_request *req, auth_session *ses
 #ifdef HAVE_SSPI
 static char *request_sspi(auth_session *sess, struct auth_request *request) 
 {
-    return ne_concat(sess->protocol->name, " ", sess->sspi_token, "\r\n", NULL);
+    if (sess->sspi_token)
+        return ne_concat(sess->protocol->name, " ", sess->sspi_token, "\r\n", NULL);
+    else
+        return NULL;
 }
 
 static int sspi_challenge(auth_session *sess, int attempt,
@@ -725,7 +728,7 @@ static int ntlm_challenge(auth_session *sess, int attempt,
     
     NE_DEBUG(NE_DBG_HTTPAUTH, "auth: NTLM challenge.\n");
     
-    if (!parms->opaque) {
+    if (!parms->opaque && (!sess->ntlm_context || (attempt > 1))) {
         char password[NE_ABUFSIZ];
 
         if (get_credentials(sess, errmsg, attempt, parms, password)) {
@@ -1464,6 +1467,14 @@ static int ah_post_send(ne_request *req, void *cookie, const ne_status *status)
     }
 #endif
 
+#ifdef HAVE_SSPI
+    /* whatever happens: forget the SSPI token cached thus far */
+    if (sess->sspi_token) {
+        ne_free(sess->sspi_token);
+        sess->sspi_token = NULL;
+    }
+#endif
+
     NE_DEBUG(NE_DBG_HTTPAUTH, 
 	     "ah_post_send (#%d), code is %d (want %d), %s is %s\n",
 	     areq->attempt, status->code, sess->spec->status_code, 
@@ -1497,8 +1508,10 @@ static int ah_post_send(ne_request *req, void *cookie, const ne_status *status)
                             sess->protocol 
                             && (sess->protocol->flags & AUTH_FLAG_CONN_AUTH));
     }
+
 #ifdef HAVE_SSPI
-    else if (sess->sspi_context) {
+    /* Whatever happens: clear the SSPI context if it exists. */
+    if (sess->sspi_context) {
         ne_sspi_clear_context(sess->sspi_context);
     }
 #endif
